package de.mutschler.rpdl.basicvalue;

public class ValueBool extends BasicValue implements java.io.Serializable
{
	static final long serialVersionUID = -537507568119658371L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** Here the actual value is stored */
	private boolean value;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	public ValueBool() {this(true);}
	
	public ValueBool (String s) {
		super(s);
		value = Boolean.getBoolean(s);
	}
	
	public ValueBool (boolean b) {
		super();
		value = b;
	}
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** The name of this type. */
	public static final String TYPENAME="BOOL";
	/** The name of this type. */
	public String getType() {return TYPENAME;}
	
	/** Return the cast operation for casting into the same value type. */
	public int getCastOp() {return BasicValue.TO_BOOL;}

	/** Get the actual value of this type. */
	public boolean getValue() {return value;}
	
	/** Get a string-representation of the value. This is used for concatenation. */
	public String getStringValue() {return (value?Boolean.TRUE:Boolean.FALSE).toString();}
	
	public void setValue(BasicValue t) throws RpdlConversionException
	{
		if (TYPENAME.equals(t.getType())) {
			value = ((ValueBool)t).getValue();
			return;
		}
		// the superclass can throw the excpeption
		super.setValue(t);
	}
	
	//----------------------------------------------------------------------
	//------- mathematical operations
	//----------------------------------------------------------------------

	/** Perform a calculation. */
	public BasicValue calculate(int op, BasicValue t) throws RpdlConversionException
	{
		String type = ((t==null)?null:t.getType());
		switch(op) {
			case TO_BOOL:
				return this;
			case OP_NEG:									// !BOOL -> BOOL
				return new ValueBool(!value);
			case COMP_EQ:
				if (ValueBool.TYPENAME.equals(type)) {		// BOOL = BOOL -> BOOL
					return new ValueBool(value == ((ValueBool)t).getValue());
				}
				break;
			case COMP_NE:
				if (ValueBool.TYPENAME.equals(type)) {		// BOOL <> BOOL -> BOOL
					return new ValueBool(value != ((ValueBool)t).getValue());
				}
				break;
		}
		return super.calculate(op, t);	// let the superclass throw the exception
	}

	/**
	 * For the static type-checking. Return the expected type for an
	 * operation.
	 */
	public String getResultType(int op, String type) {
		switch(op) {
			case TO_BOOL:	return ValueBool.TYPENAME;
			case OP_NEG:	return ValueBool.TYPENAME;		// !BOOL -> BOOL
			case COMP_EQ:
				if (ValueBool.TYPENAME.equals(type)) {		// BOOL = BOOL -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_NE:
				if (ValueBool.TYPENAME.equals(type)) {		// BOOL <> BOOL -> BOOL
					return ValueBool.TYPENAME;
				}
				break;
		}
		return super.getResultType(op, type);	// let the superclass return the error
	}
	
	
	//----------------------------------------------------------------------
	//------- overridden methods
	//----------------------------------------------------------------------
	/** Create a copy of this object */
	public Object clone() {return new ValueBool(value);}
		
}