package de.mutschler.rpdl.basicvalue;

public class ValueNumber extends BasicValue implements java.io.Serializable
{
	static final long serialVersionUID = -3141618190596272036L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** Here the actual value is stored */
	private float value;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	public ValueNumber() {this(0f);}
	
	public ValueNumber (String s) {
		super(s);
		value = Float.parseFloat(s);
	}
	
	public ValueNumber (float f) {
		super();
		value = f;
	}
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** The name of this type. */
	public static final String TYPENAME="NUMBER";
	/** The name of this type. */
	public String getType() {return TYPENAME;}
	
	/** Return the cast operation for casting into the same value type. */
	public int getCastOp() {return BasicValue.TO_NUMBER;}

	/** Get the actual value of this type. */
	public float getValue() {return value;}
	
	/** Get a string-representation of the value. This is used for concatenation. */
	public String getStringValue() {
		String s = Float.toString(value);
		boolean truncate=false;
		int i;
		for (i=s.length()-1; i>=1; i--) {
			char c = s.charAt(i);
			if (c != '0') {break;}
			if (c == '.') {i--;break;}
			truncate = true;
		}
		if (truncate)
			return s.substring(0, i);
		else
			return s;
		
	}

	public void setValue(BasicValue t) throws RpdlConversionException
	{
		if (TYPENAME.equals(t.getType())) {
			value = ((ValueNumber)t).getValue();
			return;
		}
		if (ValueInt.TYPENAME.equals(t.getType())) {
			value = ((ValueInt)t).getValue();
			return;
		}
		// the superclass can throw the excpeption
		super.setValue(t);
	}
	//----------------------------------------------------------------------
	//------- mathematical operations
	//----------------------------------------------------------------------

	/** Perform a calculation. */
	public BasicValue calculate(int op, BasicValue t) throws RpdlConversionException
	{
		String type = ((t==null)?null:t.getType());
		switch(op) {
			case TO_NUMBER:
				return this;
			case TO_INT:
				return new ValueInt((int)value);
			case OP_NEG:								// -NUMBER -> NUMBER
				return new ValueNumber(-value);
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER + INT -> NUMBER
					return new ValueNumber(value + ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER + NUMBER -> NUMBER
					return new ValueNumber(value + ((ValueNumber)t).getValue());
				}
				if (ValueString.TYPENAME.equals(type)) {	// NUMBER + STRING -> STRING
					return new ValueString(value + ((ValueString)t).getValue());
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER - INT -> NUMBER
					return new ValueNumber(value - ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER - NUMBER -> NUMBER
					return new ValueNumber(value - ((ValueNumber)t).getValue());
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER * INT -> NUMBER
					return new ValueNumber(value * ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER * NUMBER -> NUMBER
					return new ValueNumber(value * ((ValueNumber)t).getValue());
				}
				break;
			case OP_DIV:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER / INT -> NUMBER
					return new ValueNumber(value / ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER / NUMBER -> NUMBER
					return new ValueNumber(value / ((ValueNumber)t).getValue());
				}
				break;
			case COMP_EQ:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER = INT -> bool
					return new ValueBool(value == ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER = NUMBER -> bool
					return new ValueBool(value == ((ValueNumber)t).getValue());
				}
				break;
			case COMP_NE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER <> INT -> bool
					return new ValueBool(value != ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER <> NUMBER -> bool
					return new ValueBool(value != ((ValueNumber)t).getValue());
				}
				break;
			case COMP_LE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER <= INT -> bool
					return new ValueBool(value <= ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER <= NUMBER -> bool
					return new ValueBool(value <= ((ValueNumber)t).getValue());
				}
				break;
			case COMP_LT:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER < INT -> bool
					return new ValueBool(value < ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER < NUMBER -> bool
					return new ValueBool(value < ((ValueNumber)t).getValue());
				}
				break;
			case COMP_GE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER >= INT -> bool
					return new ValueBool(value >= ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER >= NUMBER -> bool
					return new ValueBool(value >= ((ValueNumber)t).getValue());
				}
				break;
			case COMP_GT:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER > INT -> bool
					return new ValueBool(value > ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER > NUMBER -> bool
					return new ValueBool(value > ((ValueNumber)t).getValue());
				}
				break;
		}
		return super.calculate(op, t);	// let the superclass throw the exception
	}

	/**
	 * For the static type-checking. Return the expected type for an
	 * operation.
	 */
	public String getResultType(int op, String type) {
		switch(op) {
			case TO_NUMBER:	return ValueNumber.TYPENAME;
			case TO_INT:	return ValueInt.TYPENAME;
			case OP_NEG:	return ValueNumber.TYPENAME;	// -NUMBER -> NUMBER
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER + INT -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER + NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueString.TYPENAME.equals(type)) {	// NUMBER + STRING -> STRING
					return ValueString.TYPENAME;
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER - INT -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER - NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER * INT -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER * NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case OP_DIV:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER / INT -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER / NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case COMP_EQ:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER = INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER = NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_NE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER <> INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER <> NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER <= INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER <= NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LT:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER < INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER < NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GE:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER >= INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER >= NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GT:
				if (ValueInt.TYPENAME.equals(type)) {	// NUMBER > INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// NUMBER > NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
		}
		return super.getResultType(op, type);	// let the superclass return the error
	}

	//----------------------------------------------------------------------
	//------- overridden methods
	//----------------------------------------------------------------------
	/** Create a copy of this object */
	public Object clone() {return new ValueNumber(value);}
		
	public String toString() {return getStringValue();}
}