package de.mutschler.rpdl.command;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.*;
import java.util.*;
import java.text.*;

public class CmdCreateAttr extends RpdlCommand implements java.io.Serializable
{
	static final long serialVersionUID = 6515967983303477302L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------


	private String name, type, comment;
	private ValueDice defValue;
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/** A create attribute command.
	 * @param name the name of the attribute
	 * @param type the type of the attribute
	 * @param def the default value. This can be null.
	 * @param comment a description about the attribute
	 */
	public CmdCreateAttr(String name, String type, ValueDice def, String comment)
	{
		this.name = name;
		this.type = type;
		this.defValue = def;
		this.comment = comment;
	}

	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	/** Perform the actions when added to the main RPDL object.
	 */
	public void add(RpdlBase main) throws RpdlException
	{
		try {
			RpdlAttributeDef attrDef= new RpdlAttributeDef(name, type, defValue, comment);
			attrDef.setOrigin(getOrigin());
			main.addAttributeDef(attrDef);
		} catch (RpdlException re) {
			re.fillOrigin(getOrigin());	// update origin if necessary
			throw re;
		}
	}

	public String toString() {
		return "CmdCreateAttr: " + new RpdlAttributeDef(name, type, defValue, comment).toString();
	}
}