package de.mutschler.rpdl.command;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import java.util.*;

public class CmdLink extends RpdlCommand implements java.io.Serializable
{
	static final long serialVersionUID = -640061574764884422L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	/** Link from this object. */
	private String objName;
	/** Link to this object. */
	private FormulaObject toObject;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	public CmdLink(String from, String to)
	{
		this(from, new FormulaObject(to));
	}

	public CmdLink(String from, FormulaObject to)
	{
		this.objName = from;
		this.toObject = to;
	}

	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------

	/** Perform the actions when added to the main
	 * RPDL object.
	 */
	public void add(RpdlBase main) throws RpdlException
	{
		try {
			RpdlObject obj   = main.getObject(objName);
			if (obj == null)
				throw new RpdlException("Object " + objName + " not defined!", getOrigin());

			// this is a bit tricky:
			// parent & parentall will throw an exception,
			// $base and $xxx will return a valid object, or null if not found
			RpdlObject toObj = toObject.getObject(main, null);
			if (toObj == null)
				throw new RpdlException("Object " + toObject.getName() + " not defined!", getOrigin());

			toObj.link(obj);
		} catch (RpdlException re) {
			re.fillOrigin(getOrigin());	// update origin if necessary
			throw re;
		} catch (NullPointerException e) {
			de.mutschler.util.Log.exception(e);
			throw new RpdlException("Object " + toObject.getName() + " is invalid in link statement!", getOrigin());
		}
	}
	
	public String toString() {
		return "CmdLink: FROM " + objName + " TO " + toObject.getName();
	}
}