package de.mutschler.rpdl.command;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.statement.Stmt;
import java.util.*;
import java.text.*;

public class CmdModifyObject extends RpdlCommand implements java.io.Serializable
{
	static final long serialVersionUID = 9111067363754856082L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	private String name;
	private List statements;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/** A create object command.
	 * @param name the name of the object
	 * @param types a list of types for the object
	 */
	public CmdModifyObject(String name, List stmts)
	{
		this.name = name;
		this.statements = stmts;
	}

	/** Set the statement-list. */
//	public void setStatements(List stmts) {statements = stmts;}

	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	/** Perform the actions when added to the main RPDL object.
	 */
	public void add(RpdlBase main) throws RpdlException
	{
		try {
			// check if the object exists
			RpdlObject ro = main.getObject(name);
			if (ro == null) {
				throw new RpdlException("Object " + name + " doesn't exist!", getOrigin());
			}
			// add the statements to the object
			ro.setOrigin(getOrigin());
			for (Iterator it = statements.iterator(); it.hasNext(); ) {
				Stmt st = (Stmt)it.next();
				st.checkStatement(main, ro);
				ro.addStatement(st);
			}
		} catch (RpdlException re) {
			re.fillOrigin(getOrigin());	// update origin if necessary
			throw re;
		}
	}

	public String toString() {
		return "CmdModifyObject: " + name;
	}
}