package de.mutschler.rpdl.entity;

import java.util.*;
import de.mutschler.util.*;
import de.mutschler.rpdl.basicvalue.BasicValue;
import de.mutschler.rpdl.*;

/**
 * This class represents a mathematical calculation
 * of two formulas
 */

public class FormulaMath extends RpdlFormula implements java.io.Serializable
{
	static final long serialVersionUID = -3875094741782519500L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	private RpdlFormula param1;
	private RpdlFormula param2;
	/** This is the type of operation to perform */
	private int operation;
	
	public FormulaMath(int op, RpdlFormula param1, RpdlFormula param2) {
		this.operation = op;
		this.param1 = param1;
		this.param2 = param2;
		// set the origin of this operation to the origin of the first parameter
		setOrigin(param1.getOrigin());
	}
	
	//----------------------------------------------------------------------
	//------- Methods for interface RpdlFormula
	//----------------------------------------------------------------------
	
	/**
	 * Perform a calculation, and return the result.
	 */
	public BasicValue calc(RpdlBase main, RpdlObject obj) throws RpdlException {
		return param1.calc(main, obj).calculate(operation, param2==null?null:param2.calc(main, obj));
	}

	/**
	 * Get the type of result.
	 */
	public String getResultType(RpdlBase main, Entity e) throws RpdlException {
		String res = BasicValue.getResultType(operation, param1.getResultType(main, e),
			param2==null?null:param2.getResultType(main, e));
		ddebug("Resulttype of formula " + toString() + " is " + res);
		return res;
	}

	/**
	 * Get a list of all dependencies for this formula.
	 * This returns an Iterator with objects of type AttrAccess.
	 */
	public Set getDependencies(RpdlBase main, RpdlObject obj) throws RpdlException {
		Set set = param1.getDependencies(main, obj);
		if (param2 != null)
			set.addAll(param2.getDependencies(main, obj));
		return set;
	}

	
	public String toString() {
		return "(" + param1 + " " + BasicValue.getOpName(operation) + " " + param2 + ")";
	}
	
}
