package de.mutschler.rpdl.entity;

import java.util.*;
import de.mutschler.util.*;
import de.mutschler.rpdl.basicvalue.BasicValue;
import de.mutschler.rpdl.*;

/**
 * This class represents a mathematical calculation
 * of two formulas
 */

public class FormulaTable extends RpdlFormula implements java.io.Serializable
{
	static final long serialVersionUID = 2363912565562003368L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	private RpdlFormula lookup;
	private String tableName;
	
	public FormulaTable(RpdlBase main, String tablename, RpdlFormula lookup) {
		this.tableName = tablename;
		RpdlTable table = getTable(main);
		if (table == null)
			error("Table not created yet.");
		this.lookup = lookup;
	}
	
	//----------------------------------------------------------------------
	//------- Methods for interface RpdlFormula
	//----------------------------------------------------------------------
	
	private transient RpdlTable table;
	private RpdlTable getTable(RpdlBase main) {
		if (table == null) {
			table = main.getTable(tableName);
		}
		return table;
	}
	
	/**
	 * Perform a calculation, and return the result.
	 */
	public BasicValue calc(RpdlBase main, RpdlObject obj) throws RpdlException {
		return getTable(main).calc(lookup.calc(main, obj), main, obj);
	}

	/**
	 * Get the type of result.
	 */
	public String getResultType(RpdlBase main, Entity e) throws RpdlException {
		return getTable(main).getType();
	}

	/**
	 * Get a list of all dependencies for this formula.
	 * This returns an Iterator with objects of type AttrAccess.
	 */
	public Set getDependencies(RpdlBase main, RpdlObject obj) throws RpdlException {
		return lookup.getDependencies(main, obj);
	}

	
	public String toString() {
		return "(table: #"+(table==null?"null":table.getName()) + "(" + lookup + "))";
	}
	
}
