package de.mutschler.rpdl.entity;

import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.*;
import de.mutschler.util.StringUtil;
import java.util.*;

public class RpdlTable extends NamedObj implements java.io.Serializable
{
	static final long serialVersionUID = 7730825861949382603L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** The type of the result. */
	private String resType;
	/** The type of the parameter. */
	private String paramType;
	/** The name of the parameter. */
	private String paramName;
	
	/** A list of ranges. */
	private List entries = new ArrayList();
	
	/** Store the Attribute definition for the parameter. */
	RpdlAttributeDef paramAttrDef;
	
	/** I need the main object for the calculation */
	private RpdlBase rmain;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	public RpdlTable(RpdlBase main, String ttype, String tname, String ptype, String pname) {
		super(tname);
		rmain = main;
		resType = ttype;
		paramType = ptype;
		paramName = pname;
		paramAttrDef = new RpdlAttributeDef(paramName, paramType);
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------
	
	/** Get the type of result. */
	public String getType() {return resType;}
	
	/** Get the parameter type. */
	public String getParamType() {return paramType;}
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** Add a range. This is inserted at the right place in the list. */
	public void addEntry(RpdlTableEntry tr) {
		entries.add(tr);
	}
	
	/** Set all ranges. */
	public void setEntries(List r) throws RpdlException {
		this.entries = r;
		for (Iterator it = r.iterator(); it.hasNext(); ) {
			checkEntry((RpdlTableEntry)it.next());
		}
	}
	
	public Iterator entries() { return entries.iterator(); }
	
	/** This method checks, if an entry is valid. Therefore the following
	 * things are checked:
	 * <ul>
	 * <li> if the formula computes valid result
	 * </ul>
	 */
	private void checkEntry(RpdlTableEntry rte) throws RpdlException {
		try {
			RpdlObject dummy = createCalcObj(null);
			RpdlFormula fml = rte.getFormula();
			String restype = fml.getResultType(rmain, dummy);

			if (!restype.equals(resType)) {
				debug("adding cast to formula: " + fml);
				fml = new FormulaMath(BasicValue.getCastOperation(resType), fml, null);
				debug("cast to formula: " + fml);
				rte.setFormula(fml);
			}
			// try again
			restype = fml.getResultType(rmain, dummy);
			
			
			
			if (!restype.equals(resType)) {
				throw new RpdlException("in table " + getName() + " the Entry '" +
										rte + "' doesn't return the correct type (" + resType + "!=" + restype,
										rte.getOrigin());
			}
		} catch (RpdlException re) {
			throw new RpdlException("Error '" + re.getMessage() + "' in entry " + rte +
									" of table '" + getName() + "'.", rte.getOrigin());
		}
	}

	/** This method creates a dummy object with the necessary attribute
	 * set to the value given as parameter. This object can be used
	 * for calculating a formula in the tableentries.
	 * @param t The value to be used. This can be null.
	 */
	private RpdlObject createCalcObj(BasicValue t) throws RpdlException {
		RpdlObject o = new RpdlObject(null, "TableTemp", null);
		RpdlAttribute attr = new RpdlAttribute(paramAttrDef, o);
		if (t != null) attr.setValue(t);
		o.addAttribute(attr);
		return o;
	}
	/**
	 * Perform a calculation.
	 */
	public BasicValue calc(BasicValue param, RpdlBase main, RpdlObject obj) throws RpdlException {
		// get the formula
		RpdlTableEntry range = null;
		for (Iterator it = entries(); it.hasNext(); ) {
			range = (RpdlTableEntry)it.next();
			if (range.inRange(param)) break;
			range = null;
		}
		if (range == null)
			throw new RpdlException("Value " + param + " is not in a range of table " + this, getOrigin());
		// perform the calculation, and return the result
		RpdlObject calcObj = createCalcObj(param);
		RpdlFormula fml = range.getFormula();
		return fml.calc(main, calcObj);
	}

	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	public String toString() {
		StringBuffer sb = new StringBuffer(100);
		sb.append("TABLE " + getType() + " " + getName());
		sb.append(" (" + paramType + " " + paramName + ")" + CR + "{" + CR);
		for (Iterator it = entries.iterator(); it.hasNext(); ) {
			sb.append(StringUtil.indent(it.next().toString()));
			sb.append(CR);
		}
		sb.append("}");
		return sb.toString();
	}
}

