package de.mutschler.rpdl.statement;

import de.mutschler.util.Log;
import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.BasicValue;

import java.util.*;
/**
 * This class implements a statement, that implements a formula.
 * Since the formula depends on other attributes, this is the base
 * for almost all other statements.
 */
public abstract class Stmt_Formula extends Stmt implements java.io.Serializable
{
	static final long serialVersionUID = -4076193935957920544L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** The Formula to evaluate. */
	private RpdlFormula formula;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	/**
	 * Create a new statement which evaluates a formula.
	 * @param formula the formula to calculate
	 */

	public Stmt_Formula(RpdlFormula formula) {
		super();
		this.formula = formula;
	}

	//----------------------------------------------------------------------
	//------- Accessors
	//----------------------------------------------------------------------

	/** Get the formula. */
	public final RpdlFormula getFormula() {return formula;}

	/** To create a new formula. e.g. add a typecast to it. */
	protected void setFormula(RpdlFormula fml) {formula = fml;}

	/**
	 * This Method checks the result of a formula
	 */
	public void checkStatement(RpdlBase base, Entity e) throws RpdlException {
		super.checkStatement(base, e);
//		if (getFormula() == null) return;
		try {
			String type = getFormula().getResultType(base, e);
			if (type.equals(BasicValue.TYPE_ERR))
				throw new RpdlException("Formula is incorrect!" +
										" Statement: '" + toString() + "'", getOrigin());
		} catch (Exception ex) {
			Log.exception(ex);
			throw new RpdlException("Error in formula " + toString() + ": "+ ex.getMessage(), getOrigin());
		}
	}
	/**
	 * This method is called, when an object is linked to the base object.
	 * add all the notifier to the objects.
	 */
/*	public void onBaseLink(RpdlBase main, RpdlObject obj, StmtHandle sh) throws RpdlException {
		// add this handle to the calcList of every Attribute of the dependencies of the formula
		Set depSet = getFormula().getDependencies(main, obj);
		Set targets = getTargets(main, obj);
		for (Iterator it=depSet.iterator(); it.hasNext(); ) {
			Object dep = it.next();
			if (dep instanceof RpdlAttribute) {
				RpdlAttribute attr = (RpdlAttribute) dep;
				if (targets.contains(attr)) continue;
				attr.addTriggerHandle(sh);
				continue;
			}
			error("unknown dependency " + dep + " in formula " + getFormula() + ", " + getFormula().getOrigin());
		}
	}

	/**
	 * This returns a set of dependencies for this statement. These are of type
	 * <ul>
	 * <li><b>RpdlObject</b>: if the statement depends on an object</li>
	 * <li><b>RpdlAttribute</b>: if the statement depends on an attribute</li>
	 * </ul>
	 */
	public Set getDependencies(RpdlBase main, RpdlObject obj) throws RpdlException {
		// first get the dependencies of the super class
		Set res = super.getDependencies(main, obj);
		// now add the dependencies of the formula
//		if (getFormula() != null)
			res.addAll(getFormula().getDependencies(main, obj));
		return res;
	}
}