package de.mutschler.rpdl.statement;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.calc.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.util.Log;

import java.util.*;

public class Stmt_ReqObj extends Stmt implements java.io.Serializable
{
	static final long serialVersionUID = 7949603718890729316L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** The error-message to display, if the formula returns false. */
	private RpdlFormula errorMessage;
	
	/** The object to access. */
	private String objectName;
	
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/**
	 * Create a new require statement.
	 * @param formula the formula to calculate
	 */

	public Stmt_ReqObj(FormulaObject obj, RpdlFormula msg) {
		super();
		objectName = obj.getName();
		errorMessage = msg;
	}

	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** Get the accessor to the object. */
	public String getObjectName() {return objectName;}
	
	/** Get the message formula. */
	public RpdlFormula getMessageFormula() {return errorMessage;}
	
	//----------------------------------------------------------------------
	//------- Methods of the superclass
	//----------------------------------------------------------------------
	/** Get the execution priority. */
	public int getPriority() {return PRI_HIGH;}	// REQUIRE should be the first command executed

	/**
	 * Excecute the statement. Here the actual calculation for statements
	 * are done. 
	 */
	public void execute(RpdlBase main, RpdlObject obj) throws RpdlException {
		super.execute(main, obj);
		// now evaluate the formula:
		RpdlObject reqObj = main.getObject(getObjectName());
		if (reqObj == null || !reqObj.inBase()) {
			RpdlException re =  new RpdlAssertionException("Required object '"+getObjectName()+"' not found: " +
					getMessageFormula().calc(main, obj).getStringValue());
			re.fillOrigin(getOrigin());
			throw re;
		}
	}

	/**
	 * This method is called, when an object is linked/unlinked to the base object.
	 * called from StmtHandle.
	 */
	public void onBaseLink(RpdlBase main, RpdlObject obj, StmtHandle sh, boolean link) throws RpdlException {
		if (link) {
			main.addCalcItem(new CalcStmt(sh));	// execute once
		}
		else {
			// unlink: nothing to do
		}
	}

	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	public String toString() {
		return "REQUIRE OBJECT (" + getObjectName() + ") " + getMessageFormula() + ";";
	}
}