package de.mutschler.rpdl.tree;

import de.mutschler.rpdl.tree.*;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import java.util.*;

/**
 * This class implements the flat scan method.
 * e.g:
 * + 1
 * | + 11
 * | | + 111
 * | + 12
 * + 2
 *   + 21
 *
 * will result in the order 1,2,11,12,21,111
 */

public class FlatOrder extends ListOrder
{
	/** create the sequential list of objects. */
	protected void updateObjectList(List list) {
		list.clear();
		debug("creating new treelist");
		// add the starting object
		list.add(getMain().getBaseObject());
		// now scan all elements
		for(int i=0; i<list.size(); i++) {
			// get the next object
			RpdlObject ro = (RpdlObject)list.get(i);
			// we should not get any null-objects, but who knows...
			if (ro == null) continue;
			// now add all subobjects to the list
			for (Iterator it = ro.getSubObjects(); it.hasNext();) {
				Object o = it.next();
				// prevent adding duplicate items, since we have
				// and acyclic graph and not a real tree
				if (!list.contains(o))
					list.add(o);
			}
		}
	}
}
