package de.mutschler.rpdl.tree;

import de.mutschler.rpdl.tree.*;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import java.util.*;

/**
 * This class implements the  scan method after a generated list.
 */

public abstract class ListOrder extends ObjectOrder
{
	
	/** caching of the tree List... */
	private List treeList = new LinkedList();
	
	/** flag if the list has changed */
	private boolean changed = true;
	
	/** Get the sequential list of objects. */
	private List getObjectList() {
		// no caching yet, so the list is always created.
		if (changed) {
			updateObjectList(treeList);
			changed = false;
		}
		return treeList;
	}
	
	/** This can be implemented in a much easier way.... */
	public Iterator walk() {return getObjectList().iterator();}
	
	/**
	 * This method should be called, to notify the treewalker of changes
	 * in the tree. Here you can update internal data.
	 */
	public void treeChanged() {
		debug("resetting treelist.");
		changed = true;	// this forces a recreating on the next access
	}
	
	/** update the sequential list of objects. */
	protected abstract void updateObjectList(List list);
	
	/**
	 * This method is for comparing two rpdl-objects in a tree.
	 * The method returns a negative int, if the first object
	 * is less than the second object.
	 */
	public int compare (RpdlObject ro1, RpdlObject ro2) {
		// both Objects must be linked to the bas object
		if (!ro1.inBase() || !ro2.inBase()) return 0;
		// get the sorted list
		List ol = getObjectList();
		int idx1 = ol.indexOf(ro1);
		int idx2 = ol.indexOf(ro2);
		if (idx1 == idx2) return 0;
		return idx1<idx2?-1:1;
	}

}